#include <Servo.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <SparkFun_VL53L5CX_Library.h>
#include "MotionDetection.h"

// Define LED pins for LED blinking
#define greenLED  49
#define redLED 5

// To communicate with the other micro-processor 
//These pins are tx and rx, which could be used to communicate using the Serial but here 
//I only need one way communication or signal so I will make pin 17 high or low on the other device so that I know what to do here.
#define pico_IN  17
#define pico_OUT 16

// Define servo objects
Servo eyeballServo;
Servo eyelidServo;

// Define GPIO pins for Servo motor control
const int eyeballPin = 0;
const int eyelidPin = 1;

SparkFun_VL53L5CX myImager;
VL53L5CX_ResultsData measurementData; // Result data class structure, 1356 byes of RAM

uint16_t previousDistances[ROWS * COLS] = {0}; // Previous frame distances
int iterationCount = 0; // Track number of iterations

// Define rotation angles and speeds
const int eyeballHome = 20;       // Set midpoint for eyeball as 90 degrees
const int eyeballRotation = 30;  // Eyeball rotation range (7 degrees each direction for subtle movement)
const int eyelidOpen = 0;        // Eyelid open position
const int eyelidClosed = 60;     // Eyelid closed position
const int moveDelay = 15;        // Delay for smoother, slower movement

//for stepper motor 
#define SLEEP 6
#define EN 7
#define DIR 8
#define STEP 9
#define M1 10
#define M0 11
#define STEPS_PER_REV 1000
#define DELAYTIME 1000
#define MICROSTEPS 1

int currentAngle = 0; // this will assume the stepper motor is in home !!!!! write code to make everything home

//hall effect sensors
#define HALL_SENSOR_BACK 2
#define HALL_SENSOR_FRONT 3

unsigned long lastDebounceTime1 = 0;
unsigned long lastDebounceTime2 = 0;
const unsigned long debounceDelay = 50; // Debounce time in milliseconds

bool sensor1State = HIGH; // Initial state (no magnet)
bool sensor2State = HIGH; // Initial state (no magnet)

void setup() {
  // Initialize Serial for debugging
  Serial.begin(9600);
  delay(500);  // Give some time for initialization

  // Initialize the LED pins as output
  pinMode(greenLED, OUTPUT);
  pinMode(redLED, OUTPUT);

  // Test LEDs briefly during setup
  digitalWrite(greenLED, HIGH);
  delay(200);
  digitalWrite(greenLED, LOW);
  digitalWrite(redLED, HIGH);
  delay(200);
  digitalWrite(redLED, LOW);

  // Initialize I2C for time-of-flight sensor
  Wire1.setSCL(19);
  Wire1.setSDA(18);
  Wire1.begin();
  Wire1.setClock(400000);

  Serial.println("Initializing sensor board. This can take up to 10s. Please wait.");
  if (myImager.begin(0x29, Wire1) == false)
  {
    Serial.println(F("Sensor not found - check your wiring. Freezing"));
  }

  myImager.setResolution(8 * 8);
  myImager.startRanging();

  // Attach servos to pins
  eyeballServo.attach(eyeballPin);
  eyelidServo.attach(eyelidPin);

  // Set initial positions
  eyeballServo.write(eyeballHome);    // Move eyeball to home position
  eyelidServo.write(eyelidOpen);      // Set eyelid to open position

  //setup for stepper motor driver
  pinMode(SLEEP,OUTPUT);
  digitalWrite(SLEEP,HIGH);

  pinMode(EN,OUTPUT);
  digitalWrite(EN,HIGH);
  
  pinMode(STEP,OUTPUT);
  digitalWrite(STEP,LOW);
  
  pinMode(DIR,OUTPUT);
  digitalWrite(DIR,LOW);

  pinMode(M0,OUTPUT);
  pinMode(M1,OUTPUT);
  digitalWrite(M0,HIGH);
  digitalWrite(M1,HIGH);

  //hall effect sensors
  pinMode(HALL_SENSOR_FRONT, INPUT_PULLUP);
  pinMode(HALL_SENSOR_BACK, INPUT_PULLUP);

  //homing the rock
  homing(); // Call the homing function
}

void loop() {
  // Check if the input from the other Pico is HIGH
  if (digitalRead(pico_IN) == HIGH) {
    Serial.println("Signal detected: Executing pre-defined movement.");
    
    // Rotate things 45 degrees left and right
    rotateToAngle(45);
    delay(500);
    rotateToAngle(-45);
    delay(500);

    // Blink the eye
    blinkEyelid();

    // Move the eyeball left and right
    moveEyeball(eyeballHome + eyeballRotation);
    delay(500);
    moveEyeball(eyeballHome - eyeballRotation);
    delay(500);

    // Blink the eye again
    blinkEyelid();
  } 
  else {
    Serial.println("No signal detected: Performing motion detection.");
    int detectedZone = detectMotion(myImager, measurementData, previousDistances, iterationCount);

    // Check if motion is detected
    if (detectedZone != -1) {
      int angle = 0; // Default angle
      
      // Map zones to specific angles
      switch (detectedZone) {
        case 1: angle = 30; break;
        case 2: angle = 15; break;
        case 3: angle = 0; break;
        case 4: angle = -15; break;
        case 5: angle = -30; break;
      }

      Serial.println("Motion detected in zone: ");
      Serial.println(detectedZone);
      Serial.println("Moving to angle: ");
      Serial.println(angle);

      // Turn in the detected direction
      rotateToAngle(angle);
      delay(500);

      // Blink the eye
      blinkEyelid();

      // Move the eyeball left and right
      moveEyeball(eyeballHome + eyeballRotation);
      delay(500);
      moveEyeball(eyeballHome - eyeballRotation);
      delay(500);

      // Blink the eye again
      blinkEyelid();
    }
  }

  delay(1000); // Small delay before repeating the loop
}

// Function to blink the on-board LED lights
void onboardleds() {
  //Serial.println("Blinking LEDs");
  digitalWrite(greenLED, HIGH); // Turn Green LED onm
  delay(200);
  digitalWrite(greenLED, LOW);  // Turn Green LED off
  delay(200);
  digitalWrite(redLED, HIGH);   // Turn RED LED on
  delay(200);
  digitalWrite(redLED, LOW);    // Turn RED LED off
  delay(200);
}

// Function to simulate smooth eyelid blinking
void blinkEyelid() {
  for (int pos = eyelidOpen; pos <= eyelidClosed; pos++) {
    eyelidServo.write(pos);  // Close eyelid slowly
    delay(moveDelay);        // Small delay for smooth motion
  }
  
  delay(200);  // Pause with closed eyelid to simulate a blink

  for (int pos = eyelidClosed; pos >= eyelidOpen; pos--) {
    eyelidServo.write(pos);  // Open eyelid slowly
    delay(moveDelay);        // Small delay for smooth motion
  }
}

// Function to simulate smooth eyeball movement
void moveEyeball(int targetPosition) {
  int currentPosition = eyeballServo.read();
  int step = (currentPosition < targetPosition) ? 1 : -1;  // Determine movement direction

  for (int pos = currentPosition; pos != targetPosition; pos += step) {
    eyeballServo.write(pos);
    delay(moveDelay);  // Delay for smoother, gradual movement
  }
}

void rotateToAngle(int targetAngle) {
  float gearRatio = 16.0 / 48.0; // Stepper to big gear ratio
  int stepsPerDegree = (STEPS_PER_REV * MICROSTEPS) / 360; // Steps per degree for stepper
  int bigGearStepsPerDegree = stepsPerDegree / gearRatio; // Steps per degree for big gear

  // Calculate target steps for the big gear
  int targetSteps = targetAngle * bigGearStepsPerDegree; // Total steps for big gear
  int currentSteps = currentAngle * bigGearStepsPerDegree; // Current position in steps
  int stepsToMove = targetSteps - currentSteps; // Steps required to reach target

  // Determine rotation direction
  if (stepsToMove > 0) {
    digitalWrite(DIR, HIGH); // Rotate clockwise
  } else {
    digitalWrite(DIR, LOW); // Rotate counter-clockwise
    stepsToMove = -stepsToMove; // Make steps positive
  }

  // Rotate the stepper motor
  for (int i = 0; i < stepsToMove; ++i) {
    digitalWrite(STEP, HIGH);
    delayMicroseconds(DELAYTIME);
    digitalWrite(STEP, LOW);
    delayMicroseconds(DELAYTIME);
  }

  currentAngle = targetAngle; // Update current position
}

void homing() {
  Serial.println("Homing process started...");

  // Define small step size for incremental rotation
  const int smallStepAngle = 5;  // Degrees to rotate per step while searching
  bool homed = false;            // Flag to indicate if homing is complete

  while (!homed) {
    int frontSensorState = digitalRead(HALL_SENSOR_FRONT); // Read front sensor
    int backSensorState = digitalRead(HALL_SENSOR_BACK);   // Read back sensor

    if (frontSensorState == LOW) { // Front sensor detected a magnet
      Serial.println("Front Hall Effect sensor detected. Rotating 45° clockwise to home.");
      rotateToAngle(45);           // Rotate 45° clockwise
      homed = true;
    } else if (backSensorState == LOW) { // Back sensor detected a magnet
      Serial.println("Back Hall Effect sensor detected. Rotating 45° counterclockwise to home.");
      rotateToAngle(-45);          // Rotate 45° counterclockwise
      homed = true;
    } else {
      // Neither sensor detected, rotate in small increments to search
      Serial.println("Searching for home position...");
      rotateToAngle(currentAngle + smallStepAngle); // Rotate a small step forward
      delay(100); // Small delay to allow for sensor stabilization
    }
  }

  Serial.println("Homing process complete. Machine is at home position.");
  currentAngle = 0; // Set current angle to home position
}
