#ifndef MOTION_DETECTION_H
#define MOTION_DETECTION_H

#include <SparkFun_VL53L5CX_Library.h>

// Constants
#define ROWS 8
#define COLS 8
#define NUM_ZONES 5
#define MAX_DISTANCE 2000 // in mm
#define MIN_DISTANCE 250  // in mm
#define MOTION_THRESHOLD 50 // in mm
#define ITERATIONS 5 // Number of iterations for motion detection

// Zones definition (using grid indices)
const int zone1[] = {0, 1, 8, 2, 3, 9, 10, 16, 17, 24};
const int zone2[] = {4, 5, 12, 13, 18, 19, 25, 26, 32, 33};
const int zone3[] = {6, 7, 14, 15, 20, 21, 22, 23, 27, 28, 29, 30, 34, 35, 36, 37, 38, 39, 40, 41, 48, 49};
const int zone4[] = {31, 42, 43, 50, 51, 52, 53, 54, 55, 56, 57};
const int zone5[] = {47, 58, 59, 60, 61, 62, 63};

// Function to check if an index is in a specific zone
bool isInZone(int index, const int *zoneArray, int zoneSize)
{
  for (int i = 0; i < zoneSize; i++)
  {
    if (index == zoneArray[i])
      return true;
  }
  return false;
}

// Function to perform motion detection
int detectMotion(SparkFun_VL53L5CX &sensor, VL53L5CX_ResultsData &measurementData, uint16_t *previousDistances, int &iterationCount)
{
  static int motionCount[NUM_ZONES] = {0}; // To store detection counts for each zone
  int zoneScores[NUM_ZONES] = {0};         // To store scores for each zone
  int zoneCounts[NUM_ZONES] = {0};         // To count points per zone

  // Poll sensor data
  if (sensor.isDataReady())
  {
    if (sensor.getRangingData(&measurementData))
    {
      for (int i = 0; i < ROWS * COLS; i++)
      {
        // Get current distance and apply filtering
        uint16_t currentDistance = measurementData.distance_mm[i];
        if (currentDistance < MIN_DISTANCE) currentDistance = MIN_DISTANCE;
        if (currentDistance > MAX_DISTANCE) currentDistance = MAX_DISTANCE;

        // Compare with previous frame
        if (previousDistances[i] > 0)
        {
          int delta = abs(currentDistance - previousDistances[i]);
          if (delta > MOTION_THRESHOLD) // Threshold for motion detection
          {
            // Assign scores to zones
            if (isInZone(i, zone1, sizeof(zone1) / sizeof(zone1[0]))) { zoneScores[0] += delta; zoneCounts[0]++; }
            else if (isInZone(i, zone2, sizeof(zone2) / sizeof(zone2[0]))) { zoneScores[1] += delta; zoneCounts[1]++; }
            else if (isInZone(i, zone3, sizeof(zone3) / sizeof(zone3[0]))) { zoneScores[2] += delta; zoneCounts[2]++; }
            else if (isInZone(i, zone4, sizeof(zone4) / sizeof(zone4[0]))) { zoneScores[3] += delta; zoneCounts[3]++; }
            else if (isInZone(i, zone5, sizeof(zone5) / sizeof(zone5[0]))) { zoneScores[4] += delta; zoneCounts[4]++; }
          }
        }

        // Update previous distances
        previousDistances[i] = currentDistance;
      }

      // Calculate average score for each zone
      float avgScores[NUM_ZONES] = {0};
      for (int z = 0; z < NUM_ZONES; z++)
      {
        if (zoneCounts[z] > 0)
          avgScores[z] = (float)zoneScores[z] / zoneCounts[z];
      }

      // Find the zone with the most motion in this iteration
      int maxZone = -1;
      float maxScore = 0;
      for (int z = 0; z < NUM_ZONES; z++)
      {
        if (avgScores[z] > maxScore)
        {
          maxScore = avgScores[z];
          maxZone = z + 1;
        }
      }

      // Increment the motion count for the most active zone
      if (maxZone != -1)
        motionCount[maxZone - 1]++;

      iterationCount++;

      // After defined iterations, determine the final zone with most motion
      if (iterationCount >= ITERATIONS)
      {
        int finalZone = -1;
        int maxMotionCount = 0;
        for (int z = 0; z < NUM_ZONES; z++)
        {
          if (motionCount[z] > maxMotionCount)
          {
            maxMotionCount = motionCount[z];
            finalZone = z + 1;
          }
        }

        // Reset for next cycle
        iterationCount = 0;
        for (int i = 0; i < NUM_ZONES; i++)
          motionCount[i] = 0;

        return finalZone; // Return the zone with the most motion
      }
    }
  }

  return -1; // No result yet
}

#endif
